import logging

from defence360agent.contracts.plugins import MessageSink
from defence360agent.utils import RecurringCheckStop, recurring_check

from im360.subsys.remoteip import ModRemoteIp

logger = logging.getLogger(__name__)

_CHECK_PERIOD = 3600


class RemoteIpInstall(MessageSink):
    async def create_sink(self, loop):
        self._loop = loop
        self._task = self._loop.create_task(self._check_remoteip())

    async def shutdown(self):
        self._task.cancel()
        await self._task

    @recurring_check(_CHECK_PERIOD)
    async def _check_remoteip(self):
        remoteip = ModRemoteIp()
        if not remoteip.is_supported():
            logger.error(
                "Automatic mod_remoteip installation is not supported"
                " on this server"
            )
            raise RecurringCheckStop()
        if not await remoteip.is_installed():
            await remoteip.install()
            logger.info("mod_remoteip successfully installed")
        await remoteip.customize_logging()
