import logging

from defence360agent.contracts.license import LicenseCLN
from defence360agent.utils import OsReleaseInfo
from im360.subsys.features.hardened_php import get_hardened_php_feature
from im360.subsys.features.invisible_captcha import InvisibleCaptcha
from im360.subsys.features.kernel_care import KernelCare

logger = logging.getLogger(__name__)


def list_feature_types():
    if OsReleaseInfo.id_like() & OsReleaseInfo.RHEL_FEDORA_CENTOS:
        return (KernelCare, get_hardened_php_feature())
    else:
        return ()


def get_applicable_features():
    return {
        "hardened-php": get_hardened_php_feature(),
        "invisible-captcha": InvisibleCaptcha,
        "kernelcare": KernelCare,
    }


def update_features_repos(op):
    """
    Features repo-files depends on server_id, so it should be actual
    after every registration operation
    :param str op: generate_repo/remove_repo
    """
    for feature in list_feature_types():
        method_to_call = getattr(feature(), op, None)
        if callable(method_to_call):
            method_to_call()


async def update_repos():
    if LicenseCLN.is_valid():
        # Update credentials in repos config
        logger.info("Updating repositories configuration with new credentials")
        update_features_repos("generate_repo")
    else:
        # Disable CLN repos for invalid license
        logger.warning(
            "License is invalid. Repositories for features will be disabled"
        )
        update_features_repos("remove_repo")
